
/*
Copyright (c) 2019 The Khronos Group Inc.
Use of this source code is governed by an MIT-style license that can be
found in the LICENSE.txt file.
*/


void main()
{
    gl_FragColor = vec4(1.0);  // can be used in fragment shader only
}
