
/*
Copyright (c) 2019 The Khronos Group Inc.
Use of this source code is governed by an MIT-style license that can be
found in the LICENSE.txt file.
*/


#ifdef GL_ES
precision mediump float;
#endif
void main()
{
    vec2 v2;
    vec3 v3;
    bool b = v2 == v3; // equal operator cannot operator on vectors of different sizes
}
