/* eslint-disable import/no-named-default, import/no-unresolved, import/named */
import { default as default_2 } from "./bug_1894631_module_2.mjs";
import { default as default_non } from "./non_existing.mjs";

module1_loaded = true;
