canvas {
  background-color: white;
  background-size: 8px 8px;
}

.shader-source {
  border: 1px dashed black;
  padding: 1em;
}

.shader-source li:nth-child(odd) { background: #f8f8f8; }
.shader-source li:nth-child(even) { background: #f0f0f0; }

.testimages {
}

.testimages br {
  clear: both;
}

.testimages > div {
  float: left;
  margin: 1em;
}

IMG {
  border: 1px solid black;
  width: 250px;
  height: 250px;
}
