var rollupBabel6EvalMaps = (function () {
  'use strict';

  function root() {
    var one = 1;

    {
      var _two = 4;
      var _three = 5;

      console.log("pause here", one, _two, _three);
    }
  }

  return root;

}());
//# sourceMappingURL=eval-maps.js.map
