window.onerror = function (e) {
  document.getElementById("result").textContent = e;
};

function f() {
  "use asm";
  function test() {
    return 10;
  }
  return test;
}

var dummy = [
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
  "dummy text to exceed the minimal source length for bytecode cache",
];
